@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2002 Bill Wilson
+|  Copyright (C) 1999-2004 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -19,25 +19,15 @@
 |  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/*
-|  5/7/2000 Applied code from Takeshi Aihana <takeshi@turbolinux.co.jp>
-|		ENABLE_NLS defines and string internationalizations code modifications.
-|		Strings in most files are touched, but I just note this here in
-|		gkrellm.h
-*/
-
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
+#if defined(HAVE_CONFIG_H)
+#include "config.h"
 #endif  
 
+#if !defined(WIN32)
 #include <sys/param.h>
+#endif
 
 #include <gtk/gtk.h>
-#include <gdk_imlib.h>
-
-/* To get at the private structure of pixmaps (for sizes)
-*/
-#include <gdk/gdkx.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -45,10 +35,13 @@
 #include <ctype.h>
 #include <fcntl.h>
 #include <time.h>
-#include <signal.h>
+
+#if !defined(WIN32)
 #include <unistd.h>
-#include <sys/ioctl.h>
-#include <sys/socket.h>
+#else
+#include "win32.h"
+#endif
+
 #include <sys/stat.h>
 
 #include <locale.h>
@@ -58,35 +51,23 @@
 #endif
 
 #if !defined(__FreeBSD__) && !defined(__linux__) && !defined(__NetBSD__) \
-	&& !defined(__OpenBSD__) && !defined(__solaris__)
+	&& !defined(__OpenBSD__) && !defined(__solaris__) && !defined(WIN32) \
+	&& !defined(__APPLE__)
 #define  USE_LIBGTOP
 #endif
 
-#if defined(USE_LIBGTOP)
-#include <glibtop.h>
-#include <glibtop/open.h>
-#include <glibtop/close.h>
-#include <glibtop/xmalloc.h>
-#include <glibtop/parameter.h>
-#include <glibtop/netload.h>
-#endif
 
+#if !defined(PACKAGE)
+#define PACKAGE	"gkrellm"
+#endif
 
 /* Internationalization support.
 */
-#if defined(ENABLE_NLS) && ! defined(PACKAGE)
-#	define PACKAGE	"gkrellm"
-#endif
-
 #if defined (ENABLE_NLS)
-#	include <libintl.h>
-#	ifdef PACKAGE
-#		undef _
-#		define _(String) dgettext(PACKAGE,String)
-#	else
-#		define _(String) gettext(String)
-#	endif	/* PACKAGE */
-#	ifdef gettext_noop
+#include <libintl.h>
+#	undef _
+#	define _(String) dgettext(PACKAGE,String)
+#	if defined(gettext_noop)
 #		define N_(String) gettext_noop(String)
 #	else
 #		define N_(String) (String)
@@ -106,57 +87,82 @@
 
 /* -------------------------------------------------------------------
 */
-#define	GKRELLM_VERSION_MAJOR	1
+#define	GKRELLM_VERSION_MAJOR	2
 #define	GKRELLM_VERSION_MINOR	2
-#define	GKRELLM_VERSION_REV		10
-/* Backward compatibility */
-#define	VERSION_MAJOR			GKRELLM_VERSION_MAJOR
-#define	VERSION_MINOR			GKRELLM_VERSION_MINOR
-#define	VERSION_REV				GKRELLM_VERSION_REV
+#define	GKRELLM_VERSION_REV		5
+#define	GKRELLM_EXTRAVERSION	""
 
-#define	GKRELLM_DIR				".gkrellm"
-#define	GKRELLM_USER_CONFIG		".gkrellm/user_config"
-#define	GKRELLM_THEME_CONFIG	".gkrellm/theme_config"
-#define GKRELLM_THEMES_DIR		".gkrellm/themes"
-#define	GKRELLM_DATA_DIR		".gkrellm/data"
-#define	GKRELLM_PLUGINS_DIR		".gkrellm/plugins"
+#define GKRELLM_CHECK_VERSION(major,minor,rev)    \
+	(GKRELLM_VERSION_MAJOR > (major) || \
+	(GKRELLM_VERSION_MAJOR == (major) && GKRELLM_VERSION_MINOR > (minor)) || \
+	(GKRELLM_VERSION_MAJOR == (major) && GKRELLM_VERSION_MINOR == (minor) && \
+	GKRELLM_VERSION_REV >= (rev)))
+
+#define	GKRELLM_DIR				".gkrellm2"
+#define	GKRELLM_USER_CONFIG		".gkrellm2/user-config"
+#define	GKRELLM_2_1_14_CONFIG	".gkrellm2/user_config"
+#define	GKRELLM_THEME_CONFIG	".gkrellm2/theme_config"
+#define GKRELLM_THEMES_DIR		".gkrellm2/themes"
+#define	GKRELLM_DATA_DIR		".gkrellm2/data"
+#define	GKRELLM_PLUGINS_DIR		".gkrellm2/plugins"
+#define	GKRELLM_LOCK_FILE		".gkrellm2/lock"
 #define GKRELLMRC				"gkrellmrc"
 
-#define	PLUGINS_INSTALL_FILE		".gkrellm/plugins_install"
-#define	PLUGINS_PLACEMENT_FILE	".gkrellm/plugins_placement"
+#define	PLUGIN_ENABLE_FILE		".gkrellm2/plugin_enable"
+#define	PLUGIN_PLACEMENT_FILE	".gkrellm2/plugin_placement"
 
-#define	LOCAL_THEMES_DIR		"/usr/local/share/gkrellm/themes"
+
+#if !defined(WIN32)
+
+#define	LOCAL_THEMES_DIR		"/usr/local/share/gkrellm2/themes"
 #if !defined(SYSTEM_THEMES_DIR)
-#define	SYSTEM_THEMES_DIR		"/usr/share/gkrellm/themes"
+#define	SYSTEM_THEMES_DIR		"/usr/share/gkrellm2/themes"
+#endif
+#define	LOCAL_PLUGINS_DIR		"/usr/local/lib/gkrellm2/plugins"
+#if !defined(SYSTEM_PLUGINS_DIR)
+#define	SYSTEM_PLUGINS_DIR		"/usr/lib/gkrellm2/plugins"
 #endif
 
-#define	LOCAL_PLUGINS_DIR		"/usr/local/lib/gkrellm/plugins"
-#if !defined(SYSTEM_PLUGINS_DIR)
-#define	SYSTEM_PLUGINS_DIR		"/usr/lib/gkrellm/plugins"
+#else
+
+#define     LOCAL_THEMES_DIR		"./themes"
+#if !defined(SYSTEM_THEMES_DIR)
+#define     SYSTEM_THEMES_DIR		"./themes"
 #endif
-#define	OLD_LOCAL_PLUGINS_DIR	"/usr/local/share/gkrellm/plugins"
-#define	OLD_SYSTEM_PLUGINS_DIR	"/usr/share/gkrellm/plugins"
+#define     LOCAL_PLUGINS_DIR		"./plugins"
+#if !defined(SYSTEM_PLUGINS_DIR)
+#define     SYSTEM_PLUGINS_DIR		"./plugins"
+#endif
+
+#endif
+
 
 #define	ON				1
 #define	OFF				0
 
 #define	CFG_BUFSIZE		384
 
+
+  /* Features
+  */
+#define	GKRELLM_HAVE_THEME_SCALE			1
+#define	GKRELLM_HAVE_DECAL_TEXT_INSERT		1
+#define	GKRELLM_HAVE_CLIENT_MODE_PLUGINS 	1
+#define	GKRELLM_USING_PANGO					1
+#define	GKRELLM_HAVE_DECAL_SCROLL_TEXT		1
+
+
   /* Label midpoints are positioned as a percent of chart_width.
   */
 #define	GKRELLM_LABEL_NONE		-1
 #define	GKRELLM_LABEL_CENTER	50
 #define GKRELLM_LABEL_MAX		100
-  /* Pre 1.2.9 compatibility defines */
-#define	LABEL_NONE		GKRELLM_LABEL_NONE
-#define	LABEL_CENTER	GKRELLM_LABEL_CENTER
-#define LABEL_MAX		GKRELLM_LABEL_MAX
 
 #define	GRID_MODE_NORMAL		0
 #define	GRID_MODE_RESTRAINED	1
 
 
-  /* Decals in the decal_misc_image.
+  /* GkrellmDecals in the decal_misc_piximage.
   */
 #define	D_MISC_BLANK			0
 #define	D_MISC_AC				1
@@ -190,51 +196,66 @@ typedef struct
 			divisor;
 	gchar	*format;
 	}
-	size_abbrev_table;
+	GkrellmSizeAbbrev;
 
 
   /* Sensor types so CPU and Proc monitors  can get info from the Sensors
   |  module.  The sensor module readings for temperature and fan are
   |  reported on CPU and Proc panels.
   */
-#define SENSOR_TEMPERATURE  1
-#define SENSOR_FAN          2
-#define SENSOR_VOLTAGE      4
+#define SENSOR_TEMPERATURE  0
+#define SENSOR_FAN          1
+#define SENSOR_VOLTAGE      2
 
+#define	SENSOR_GROUP_MAINBOARD	0
+#define	SENSOR_GROUP_DISK		1
 
-#ifdef CLK_TCK
-#define	CPU_TICKS_PER_SECOND	CLK_TCK
-#else
-#define	CPU_TICKS_PER_SECOND	100		/* XXX */
-#endif
 
 #define	FULL_SCALE_GRIDS		5
 
 
 typedef struct
 	{
-	GdkFont		*font;
-	GdkFont		**font_seed;
+	PangoFontDescription *font;
+	PangoFontDescription **font_seed;
+	gpointer	privat;
 	gint		effect;
 	gint		internal;
+	gint		flags;
 	GdkColor	color;
 	GdkColor	shadow_color;
 	}
-	TextStyle;
+	GkrellmTextstyle;
 
+typedef struct
+	{
+	gchar				*text;
+	gint				x_off,
+						y_off;
+	gint				x_off_prev;		/* Kludge to check if d->x_off was */
+										/* modified directly.		*/
+	gpointer			privat;
+	GkrellmTextstyle	text_style;
+	}
+	GkrellmText;
 
-
-  /* Values for Decal flags
+  /* Values for GkrellmDecal flags
   */
-#define DF_LOCAL_PIXMAPS	0x1
-#define DF_TOP_LAYER		0x2
-#define	DF_MOVED			0x4
+#define DF_TOP_LAYER			0x1
+#define	DF_MOVED				0x2
+#define DF_LOCAL_PIXMAPS		0x4
+#define	DF_OVERLAY_PIXMAPS		0x8
+#define	DF_TEXT_OVERLAPS		0x10
+#define	DF_SCROLL_TEXT_DIVERTED	0x20
+#define	DF_SCROLL_TEXT_CENTER	0x40
+#define	DF_SCROLL_TEXT_H_LOOP	0x80
+#define	DF_SCROLL_TEXT_V_LOOP	0x100
+#define	DF_TEXT_USE_MARKUP		0x200
 
-  /* Values for Decal state
+  /* Values for GkrellmDecal state
   */
 #define	DS_INVISIBLE	0
-#define	DS_ERASING		1
-#define	DS_VISIBLE		2
+#define	DS_VISIBLE		1
 
   /* A decal is a pixmap or a part of a pixmap drawn on a panel.  The
   |  pixmap can be a graphic, a vertical stack of graphics, or a drawn
@@ -250,36 +271,45 @@ typedef struct
 	gint		x, y;		/* Position of decal in a drawable			*/	
 	gshort		flags,
 				state;
-	gint		value,		/* Index into stack, text value, etc	*/
-				modified;
+	gint		value;		/* Index into stack, text value, etc	*/
+	gboolean	modified;
 	gint		x_off,
-				y_baseline;
-	TextStyle	text_style;	/* Used if decal is a drawn text string		*/
+				y_ink;			/* Pixels from top of Pango glyph to 1st ink*/
 	gint		x_old, y_old;	/* Used if decal was moved */
+	GkrellmTextstyle text_style;	/* Used if decal is a drawn text string	*/
+	gpointer	panel;
+	GList		*text_list;
+	gchar		*scroll_text;
+	gint		scroll_width,
+				scroll_height,
+				y_off;
+
+	guint		chart_sequence_id;	/* For drawing text decal on a chart */
+	GdkRectangle ink;
 	}
-	Decal;
+	GkrellmDecal;
 
   /* Get the first decal in a panel decal list.  Use if only one decal in list.
   */
-#define	DECAL(p)	((Decal *)(((p)->decal)->data))
+#define	DECAL(p)	((GkrellmDecal *)(((p)->decal_list)->data))
 
 
 
 typedef struct
 	{
 	gchar		*string;
-	gint		lbearing,
-				rbearing,
+	gpointer	privat;
+	gint		old1,		/* olds are leftovers from GdkFont->Pango */
+				old2,
 				width,
-				ascent,
-				descent;
+				height,
+				old3;
 	gint		position;		/* 0 - 100 %	*/
-	gint		x_panel;		/* x position of label start in panel */
-	gint		y_baseline;
-	gint		h_panel;
+	gint		x_panel,		/* x position of label start in panel */
+				y_panel,
+				h_panel;
 	}
-	Label;
-
+	GkrellmLabel;
 
 
 typedef struct
@@ -291,46 +321,60 @@ typedef struct
 	gint		w;
 	gint		h;
 	}
-	Draw_rec;
+	GkrellmDrawrec;
 
 
+typedef struct
+	{
+	gint		left,
+				right,
+				top,
+				bottom;
+	}
+	GkrellmMargin;
+
+typedef struct
+	{
+	gint		left,
+				right,
+				top,
+				bottom;
+	}
+	GkrellmBorder;
+
+
+typedef struct
+	{
+	GdkPixbuf	*pixbuf;
+	GkrellmBorder border;
+	}
+	GkrellmPiximage;
+
 
   /* Bit flags for setting styles in config.c.  Also used as override flags.
   */
-#define	SET_KRELL_YOFF			1
-#define	SET_KRELL_EXPAND		2
-#define	SET_KRELL_X_HOT			3
-#define	SET_KRELL_EMA_PERIOD	4
-#define	SET_KRELL_DEPTH			5
-#define	SET_KRELL_LEFT_MARGIN	6
-#define	SET_KRELL_RIGHT_MARGIN	7
-#define	SET_LABEL_POSITION		8
-#define	SET_MARGIN				9
-#define	SET_MARGINS				10
-#define	SET_LEFT_MARGIN			11
-#define	SET_RIGHT_MARGIN		12
-#define	SET_TOP_MARGIN			13
-#define	SET_BOTTOM_MARGIN		14
-#define	SET_TEXTCOLOR_A			15
-#define	SET_TEXTCOLOR_B			16
-#define	SET_TEXTFONT_A			17
-#define	SET_TEXTFONT_B			18
-#define	SET_BORDER				19
-#define	SET_TRANSPARENCY		20
+#define	GKRELLMSTYLE_KRELL_YOFF			0x1
+#define	GKRELLMSTYLE_KRELL_LEFT_MARGIN	0x2
+#define	GKRELLMSTYLE_KRELL_RIGHT_MARGIN	0x4
+#define	GKRELLMSTYLE_KRELL_EXPAND		0x8
+#define	GKRELLMSTYLE_KRELL_X_HOT		0x10
+#define	GKRELLMSTYLE_KRELL_DEPTH		0x20
+#define	GKRELLMSTYLE_KRELL_EMA_PERIOD	0x40
+#define	GKRELLMSTYLE_LABEL_POSITION		0x80
+#define	GKRELLMSTYLE_LEFT_MARGIN		0x100
+#define	GKRELLMSTYLE_RIGHT_MARGIN		0x200
+#define	GKRELLMSTYLE_TOP_MARGIN			0x400
+#define	GKRELLMSTYLE_BOTTOM_MARGIN		0x800
+#define	GKRELLMSTYLE_TEXTCOLOR_A		0x1000
+#define	GKRELLMSTYLE_TEXTCOLOR_B		0x2000
+#define	GKRELLMSTYLE_TEXTFONT_A			0x4000
+#define	GKRELLMSTYLE_TEXTFONT_B			0x8000
+#define	GKRELLMSTYLE_BORDER				0x10000
+#define	GKRELLMSTYLE_TRANSPARENCY		0x20000
+#define	GKRELLMSTYLE_KRELL_YOFF_NOT_SCALABLE 0x40000
+#define	GKRELLMSTYLE_SCROLL_TEXT_CACHE_OFF	 0x80000
+#define	GKRELLMSTYLE_LABEL_YOFF			0x100000
 
-#define	TS_A		1
-#define	TS_B		2
-
-#define BIT(a)	(1 << (a))
-
-typedef struct
-	{
-	gint	left,
-			right,
-			top,
-			bottom;
-	}
-	Margin;
 
   /* Some of these style entries do not apply to all monitors.
   */
@@ -343,27 +387,28 @@ typedef struct
 				krell_expand,
 				krell_x_hot,
 				krell_ema_period,
-				krell_depth,
-				krell_spare;
-	gint		label_position,
-				margin,			/* 1.2.9 deprecated, use margins */
-				spare;
-	TextStyle	label_tsA;
-	TextStyle	label_tsB;
-
-	GdkImlibBorder
-				border;			/* Border for background panel image */
-	gint		transparency;
-	gboolean	themed;			/* True if theme has custom assignments */
-	Margin		margins;
+				krell_depth;
 	gint		krell_left_margin,
 				krell_right_margin;
+	gboolean	krell_yoff_not_scalable;
+	gint		label_yoff;
+	gint		label_position;
+	gboolean	scroll_text_cache_off;
+	gint		spare0;
+
+	gboolean	transparency;
+	gint		themed;			/* Non zero if theme has custom assignments */
+
+	GkrellmBorder border;			/* Border for background panel image */
+	GkrellmMargin margin;
+	GkrellmTextstyle label_tsA;
+	GkrellmTextstyle label_tsB;
 	}
-	Style;
+	GkrellmStyle;
 
 
 
-  /* The Panel of each Chart or Meter can have a moving indicator
+  /* The GkrellmPanel of each GkrellmChart or Meter can have a moving indicator
   |  representing a current sampled data value as a fraction of some full
   |  scale value.  Since there can be quite a few of them dancing around on
   |  a maxed out GKrellM display, I call them Krells - inspired by the
@@ -378,17 +423,19 @@ typedef struct
 #define KRELL_EXPAND_RIGHT_SCALED		5
 #define	KRELL_EXPAND_BAR_MODE_SCALED	6
 
+  /* Krell flags */
+#define	KRELL_FLAG_BOTTOM_MARGIN	1
+
 typedef struct
 	{
 	GdkPixmap	*pixmap;
 	GdkBitmap	*mask;
 	gint		w, h;	/* Of the full Krell - all the frames included */
-	gint		expand;	/* How this Krell was preprocessed in config.c */
 	gint		depth;	/* How many vertical frames in the krell image */
+	gint		flags;
 
 	GdkBitmap	*stencil;
-	gint		h_stencil;	/* Height of one frame + y0 offset	*/
-							/* Include y0 here so no need to set_clip_origin */
+	gint		spare0;
 	gint		reading;
 	gint		last_reading;
 	gint		full_scale;
@@ -402,8 +449,8 @@ typedef struct
 
 	gint		x0,		/* Offset in Panel to start of Krell scale range */
 				y0;
-	gint		w_scale;		/* Width of active range. */
-	gint		h_frame;		/* Height of one frame */
+	gint		w_scale;	/* Width of active range, function of margins */
+	gint		h_frame;	/* Height of one frame */
 
 	gint		ema;		/* exponential moving average */
 	gint		period;		/* of ema */
@@ -411,51 +458,74 @@ typedef struct
 	gboolean	monotonic;
 	gint		bar_mode;
 
-	Draw_rec	old_draw;	/* Last draw for restoring background.	*/
-	Draw_rec	draw;		/* Parameters to currently drawn krell	*/
+	gpointer	panel;
+	GkrellmDrawrec	old_draw;	/* Last draw for restoring background.	*/
+	GkrellmDrawrec	draw;		/* Parameters to currently drawn krell	*/
 	}
-	Krell;
+	GkrellmKrell;
 
   /* Get the first krell in a panel krell list.  Use if only one krell in list.
   */
-#define	KRELL(p)	((Krell *)(((p)->krell)->data))
+#define	KRELL(p)	((GkrellmKrell *)(((p)->krell_list)->data))
 
 typedef struct
 	{
 	GtkWidget	*hbox;			/* Container box this area is packed into */
-	GtkWidget	*fixed;
+	GdkPixmap	*bg_text_layer_pixmap;  /* A bg_pixmap with text decals */
 	GtkWidget	*drawing_area;
 	GdkPixmap	*pixmap;		/* Expose pixmap */
 	GdkBitmap	*bg_mask;
 	GdkPixmap	*bg_pixmap;		/* Bg of panel, may be dirtied with label */
-
-	Label		*label;
-	TextStyle	*textstyle;
-	GList		*decal;			/* Linked list of decals for this panel	*/
-	GList		*krell;			/* Linked list of krells for this panel	*/
-	Style		*style;
-	gint		modified;		/* True if decal/krell modified.		*/
-	gint		x, y, w, h;
 	GdkPixmap	*bg_clean_pixmap;	/* A no label bg_pixmap				*/
+	GkrellmPiximage *bg_piximage;
+
+	GkrellmLabel *label;
+	GkrellmTextstyle *textstyle;
+	GkrellmStyle *style;
+	gpointer	monitor;
+	gpointer	privat;
+	GList		*decal_list;
+	GList		*krell_list;
+	GList		*button_list;
+	gint		x, y, w, h;
+	gint		h_configure;
+
+	PangoLayout	*layout;
 
 	gint		transparency;
+	gboolean	modified;		/* True if decal/krell modified.	*/
 	gboolean	keep_lists;
 	gboolean	shown;
-	gboolean	bg_image_override;
-	GdkImlibImage *bg_image;
-	gpointer	monitor;
+	gboolean	need_decal_overlap_check;
+	gboolean	label_on_top_of_decals;
+	gboolean	scroll_text_cache_off;
+	gboolean	bg_piximage_override;
+	gboolean	button_signals_connected;
+	gint		id_press,		/* Signals if buttons in this panel	*/
+				id_release,
+				id_enter,
+				id_leave;
+	gint		y_mapped;
 	}
-	Panel;
+	GkrellmPanel;
+
+
+typedef struct
+	{
+	void		(*func)();
+	gpointer	data;
+	}
+	GkrellmCallback;
 
 
 #define CHART_WIDTH_MAX	1000
 #define CHART_WIDTH_MIN	25
 
-  /* Each chart must have a ChartConfig struct associated with it.
+  /* Each chart must have a GkrellmChartconfig struct associated with it.
   */
 #define	GKRELLM_CHARTCONFIG_KEYWORD	"chart_config"
 
-  /* Values for ChartConfig flags */
+  /* Values for GkrellmChartconfig flags */
 #define	NO_CONFIG_AUTO_GRID_RESOLUTION	1
 #define	NO_CONFIG_FIXED_GRIDS			2
 
@@ -500,13 +570,15 @@ typedef struct
 	gpointer	cb_fixed_grids_data;
 	GtkWidget	*fixed_grids_spin_button;
 
-	GList		*cd_list;			/* Same list as in parent Chart */
+	GList		*cd_list;			/* Same list as in parent GkrellmChart */
 	GList		**chart_cd_list;
-	}
-	ChartConfig;
 
-  /* Charts are drawn in layers and each data value drawn has its own layer
-  |  (the ChartData struct -> image/color of the drawn data and
+	gboolean	cb_block;
+	}
+	GkrellmChartconfig;
+
+  /* GkrellmCharts are drawn in layers and each data value drawn has its own
+  |  layer (the GkrellmChartdata struct -> image/color of the drawn data and
   |  the data colored grid lines).  Each layer is a chart sized pixmap
   |  with grid lines drawn on it.  A chart is drawn by drawing the
   |  background followed by stenciling each data layer through a
@@ -518,22 +590,46 @@ typedef struct
 	GdkPixmap	**src_pixmap;		/* The layer source image/color		*/
 	GdkPixmap	*grid_pixmap;		/* The grid pixmap to draw on src 	*/
 	}
-	ChartLayer;
+	GkrellmChartlayer;
+
 
 typedef struct
 	{
-	GtkWidget	*hbox;			/* Container box this chart is packed into */
+	GtkWidget		*vbox,
+					*image;
+	GkrellmPiximage	*piximage;
+	GdkPixmap		*pixmap,
+					*clean_pixmap;
+	GdkBitmap		*mask;
+	gint			height;			/* Width is _GK.chart_width */
+	}
+	GkrellmSpacer;
+
+typedef struct
+	{
+	gchar			*text;
+	gint			x, y;
+	gint			w, h;
+	gint			baseline,
+					y_ink;
+	gboolean		cache_valid;
+	}
+	GkrellmTextRun;
+
+typedef struct
+	{
+	GtkWidget	*box;			/* Container box this chart is packed into */
 	GtkWidget	*drawing_area;
 
-	Panel		*panel;
-	GList		*cd_list;		/* Same cd_list as in ChartConfig struct */
+	GkrellmPanel *panel;
+	GList		*cd_list;		/* Same cd_list as in GkrellmChartconfig struct */
 	gint		cd_list_index;
 	void		(*draw_chart)(gpointer);
 	gpointer	draw_chart_data;
 
-	gint		x, y, w, h;		/* h tracks h in ChartConfig	*/
+	gint		x, y, w, h;		/* h tracks h in GkrellmChartconfig	*/
 
-	ChartConfig	*config;
+	GkrellmChartconfig	*config;
 	gboolean	shown;
 	gboolean	redraw_all;
 
@@ -556,28 +652,46 @@ typedef struct
 
 	GdkPixmap	*pixmap;			/* The expose pixmap.			*/
 
-	GdkImlibImage
-				*bg_image,
-				*bg_grid_image;
+	GkrellmPiximage
+				*bg_piximage,
+				*bg_grid_piximage;
 	GdkPixmap	*bg_pixmap,			/* Working bg with grid lines	*/
 				*bg_src_pixmap,		/* bg with no grid lines		*/
 				*bg_grid_pixmap,	/* This + bg_src_pixmap builds bg_pixmap*/
-				*bg_clean_pixmap;	/* This + trans builds bg_src_pixmap */
+				*bg_clean_pixmap,	/* This + trans builds bg_src_pixmap */
+				*bg_text_pixmap;	/* Pango rendered chart text layer */
 	GdkBitmap	*bg_mask;
 	gint		transparency;
-	gboolean	bg_image_override;
+	gboolean	bg_piximage_override;
+	guint		bg_sequence_id;
+
+	gchar		*text_format_string;
+	GList		*text_run_list;		/* Caching chart text runs */
+	gint		h_text,
+				y_ink,
+				baseline_ref;
+	guint		text_run_sequence_id;
+	gboolean	text_format_reuse;
+
+	gint		y_mapped,
+				auto_recalibrate_delay;
 
 	GtkWidget	*config_window;
-	gpointer	*monitor;
+
+	GkrellmSpacer	top_spacer,
+					bottom_spacer;
+
+	GkrellmStyle	*style;
+	gpointer		monitor;
 	}
-	Chart;
+	GkrellmChart;
 
 
-  /* Values for ChartData draw_style */
+  /* Values for GkrellmChartdata draw_style */
 #define	CHARTDATA_IMPULSE	0
 #define	CHARTDATA_LINE		1
 
-  /* Values for ChartData flags */
+  /* Values for GkrellmChartdata flags */
 #define CHARTDATA_ALLOW_HIDE			1
 #define CHARTDATA_NO_CONFIG_DRAW_STYLE	2
 #define CHARTDATA_NO_CONFIG_INVERTED	4
@@ -588,13 +702,13 @@ typedef struct
 	{
 	gint		flags;
 
-	Chart		*chart;			/* Chart this data belong to */
+	GkrellmChart *chart;			/* GkrellmChart this data belong to */
 	gchar		*label;
 	gulong		current,
 				previous,
 				discard;
 	gboolean	monotonic;
-	ChartLayer	layer;			/* The image + grid for this data layer */
+	GkrellmChartlayer layer;	/* The image + grid for this data layer */
 	GdkBitmap	*data_bitmap;	/* Draw data here, use as clipmask for layer*/
 	gint		*data;
 
@@ -609,34 +723,33 @@ typedef struct
 
 	gint		y_p,
 				y_pp;
+	gboolean	y_p_valid;
 	gint		y,				/* Each data layer draws at an offset y and */
-				h,				/* within height h of its parent Chart	*/
+				h,				/* within height h of its parent GkrellmChart*/
 				w;				/* Width of data allocated.				*/
 	gint		maxval;
 	}
-	ChartData;
+	GkrellmChartdata;
 
 
 typedef struct
 	{
-	Panel		*p;
-	Decal		*d;
+	GkrellmPanel *panel;
+	GkrellmDecal *decal;
 	void		(*cb_button_click)();
-	gint		(*cb_in_button)();
 	gpointer	data;
+	gint		(*cb_in_button)();
+	gpointer	in_button_data;
+	gpointer	privat;
 	gint		cur_index;
 	gint		pressed_index;
 	gint		saved_index;
 	gint		sensitive;
 	gint		type;
-	gint		id_press,
-				id_release,
-				id_enter,
-				id_leave;
 	void		(*cb_button_right_click)();
 	gpointer	right_data;
 	}
-	DecalButton;
+	GkrellmDecalbutton;
 
 
 
@@ -645,16 +758,44 @@ typedef struct
 	gchar		*command;
 	gint		type;
 	gint		pad;
-	FILE		*pipe;		/* I read the output of some commands */
-	DecalButton	*button;
+	FILE		*pipe;		/* Read the output of some commands */
+	GkrellmDecalbutton	*button;
 	GtkTooltips	*tooltip;
 	gchar		*tooltip_comment;
+	GkrellmDecal *decal;		/* Used if DECAL_LAUNCHER type	*/
+	GkrellmMargin margin;
+
+	GtkWidget	*widget;
 	}
-	Launcher;
+	GkrellmLauncher;
 
 
 /* ------- Alerts ------- */
-#define	GKRELLM_ALERTCONFIG_KEYWORD	"alert_config"
+#define	GKRELLM_ALERTCONFIG_KEYWORD			"alert_config"
+
+typedef struct
+	{
+	struct _GkrellmMonitor	*mon;
+	gchar		*name,
+				*tab_name;
+	void		(*warn_func)(),
+				(*alarm_func)(),
+				(*update_func)(),
+				(*check_func)(),
+				(*destroy_func)();
+	void		(*config_create_func)(),
+				(*config_apply_func)(),
+				(*config_save_func)(),
+				(*config_load_func)();
+	}
+	GkrellmAlertPlugin;
+
+typedef struct
+	{
+	GkrellmAlertPlugin	*alert_plugin;
+	gpointer			data;
+	}
+	GkrellmAlertPluginLink;
 
 typedef struct
 	{
@@ -662,14 +803,16 @@ typedef struct
 				alarm_on;
 	gfloat		warn_limit,
 				alarm_limit;
+	gint		warn_delay,
+				alarm_delay;
 	GtkWidget	*warn_limit_spin_button,
 				*alarm_limit_spin_button;
 	}
-	Trigger;
+	GkrellmTrigger;
 
 typedef struct
 	{
-	Decal		*decal;
+	GkrellmDecal *decal;
 	gint		x, y, w, h;
 	GdkPixmap	*pixmap;
 	GdkBitmap	*mask;
@@ -677,22 +820,22 @@ typedef struct
 				nframes,
 				dir;
 	}
-	AlertDecal;
+	GkrellmAlertdecal;
 
 typedef struct
 	{
-	Krell	*krell;
-	gint	krell_position;
+	GkrellmKrell *krell;
+	gint		krell_position;
 	}
-	AlertKrell;
+	GkrellmAlertkrell;
 
 typedef struct
 	{
-	Panel		*panel;
+	GkrellmPanel *panel;
 	gchar		*name,
 				*unit_string;
-	AlertDecal	ad;
-	AlertKrell	ak;
+	GkrellmAlertdecal ad;
+	GkrellmAlertkrell ak;
 
 	gboolean	activated,
 				freeze,
@@ -700,16 +843,14 @@ typedef struct
 				suppress_command,
 				check_low,
 				check_high,
-				check_boolean;
+				check_hardwired;
 	gchar		*warn_command,
-				*alarm_command,
-				*voice_string;
+				*alarm_command;
 	gint		warn_repeat_set,
 				warn_repeat,
 				alarm_repeat_set,
-				alarm_repeat,
-				voice_repeat_set,
-				voice_repeat;
+				alarm_repeat;
+	gint		delay;
 
 	void		(*cb_trigger)();
 	gpointer	cb_trigger_data;
@@ -717,46 +858,65 @@ typedef struct
 	gpointer	cb_stop_data;
 	void		(*cb_config)();
 	gpointer	cb_config_data;
+	void		(*cb_config_create)();
+	gpointer	cb_config_create_data;
+	void		(*cb_command_process)();
+	gpointer	cb_command_process_data;
 
 	GtkWidget	*config_window,
 				*warn_command_entry,
 				*alarm_command_entry,
-				*voice_string_entry,
 				*warn_repeat_spin_button,
 				*alarm_repeat_spin_button,
-				*voice_repeat_spin_button;
+				*delay_spin_button,
+				*delete_button,
+				*icon_box;
+	gboolean	do_alarm_command,
+				do_warn_command;
 
-	gfloat		max_high,		/* Adjustment values */
+	gfloat		max_high,		/* limit adjustment values */
 				min_low,
 				step0,
 				step1;
 	gint		digits;
 
-	Trigger		low,
+	gint		delay_high,		/* delay adjustment values */
+				delay_low,
+				delay_step;
+
+	GkrellmTrigger low,
 				high;
+
+	gboolean	config_closing,
+				config_modified;
+
+	gchar		*id_string;		/* For unique alert names for alert plugins */
+	GList		*plugin_list;
 	}
-	Alert;
+	GkrellmAlert;
 
 /* ------------------------ */
 
 
-/* ================= Theme Config ==================*/
+#define	GKRELLM_SPACER_CHART	0
+#define	GKRELLM_SPACER_METER	1
 
 #define	CHART_PANEL_TYPE	0
 #define	METER_PANEL_TYPE	1
 #define	N_PANEL_TYPES		2
 
+#define	PANEL_LAUNCHER		CHART_PANEL_TYPE
+#define	METER_LAUNCHER		METER_PANEL_TYPE
+#define	DECAL_LAUNCHER		3
+
 #define	DEFAULT_STYLE_ID	0
-/* Old one */
-#define	DEFAULT_STYLE		0
 
 
-
-/* Style names for the builtin monitors.  Define them globally so plugins
+/* GkrellmStyle names for the builtin monitors.  Define them globally so plugins
 |  can lookup a builtin style_id.  These names are used as subdirectory
 |  names under the current theme where monitor specific images are located.
-|  They also are used in the Style lines in the gkrellmrc
-|		(eg. StyleMeter  cpu.textcolor ....
+|  They also are used in the GkrellmStyle lines in the gkrellmrc
+|		(eg. GkrellmStyleMeter  cpu.textcolor ....
 */
 #define	CPU_STYLE_NAME			"cpu"
 #define	DISK_STYLE_NAME			"disk"
@@ -768,19 +928,20 @@ typedef struct
 #define SWAP_STYLE_NAME			"swap"
 #define	FS_STYLE_NAME			"fs"
 #define	MAIL_STYLE_NAME			"mail"
-#define	APM_STYLE_NAME			"apm"
+
+  /* APM monitor is now named Battery, but don't want to break themes. */
+#define	BATTERY_STYLE_NAME		"apm"
 #define UPTIME_STYLE_NAME		"uptime"
 #define	CLOCK_STYLE_NAME		"clock"
 #define	CAL_STYLE_NAME			"cal"
 #define	HOST_STYLE_NAME			"host"
 #define	TIMER_STYLE_NAME		"timer"
 
-#define	GKRELLM_SM_ARGC_MAX		16
 
-struct	ThemeConfig
+typedef struct
 	{
-	gint		timer_ticks;
-	gint		second_tick,
+	gint		timer_ticks,
+				second_tick,
 				two_second_tick,
 				five_second_tick,
 				ten_second_tick,
@@ -788,189 +949,15 @@ struct	ThemeConfig
 				hour_tick,
 				day_tick;
 
-	gint		debug;
-	gint		spare;
-	gint		debug_level;
-	gint		demo;
-	gint		test;
+	}
+	GkrellmTicks;
 
-	gint		up_minutes;
-	gint		base_uptime;
-	time_t		start_time;
-	gint		cpu_sys_activity;
-	gint		sensor_temp_files;
-
-	gboolean	initialized;
-	gint		max_chart_height;
-	gint		monitor_height;
-
-	gint		sm_argc;
-	gchar		*sm_argv[GKRELLM_SM_ARGC_MAX];
-	gchar		*theme_path;
-	gchar		*config_suffix;		/* Overrides host_configs below */
-	gchar		*command_line_theme;
-	gchar		*command_line_plugin;
-	gint		force_host_config;
-	gint		found_host_config;
-	gint		no_config;
-
-	gint		frame_left_width,
-				frame_right_width,
-				frame_top_height,
-				frame_bottom_height;
-	gint		chart_width_ref;
-
-	gint		chart_history_length;
-	gint		chart_height_min;
-	gint		chart_height_max;
-	gint		allow_scaling;
-	gboolean	config_modified;
-	gboolean	any_transparency;
-
-	gint		update_HZ;
-	gint		chart_width;
-	gboolean	save_position;
-	gboolean	large_info_font;
-	gboolean	track_gtk_rcfiles;
-
-	gboolean	enable_hostname,		/* No separate hostname config. */
-				hostname_short,
-				enable_system_name;
-
-	gint		rx_led_x;				/* Move these to net monitor */
-	gint		rx_led_y;
-	gint		tx_led_x;
-	gint		tx_led_y;
-
-	Style		*krell_slider_style,
-				*krell_mini_style;
-
-	GdkImlibImage
-				*frame_top_image,
-				*frame_bottom_image,
-				*frame_left_image,
-				*frame_right_image;
-
-	GdkImlibImage
-				*button_panel_out_image,
-				*button_panel_in_image,
-				*button_meter_out_image,
-				*button_meter_in_image;
-
-	GdkImlibImage
-				*bg_slider_image[N_PANEL_TYPES];
-
-	GdkImlibImage
-				*krell_slider_image,
-				*krell_mini_image;
-
-	GdkImlibBorder
-				bg_slider_border[N_PANEL_TYPES];
-
-	GdkImlibBorder
-				frame_top_border,
-				frame_bottom_border,
-				frame_left_border,
-				frame_right_border,
-				button_panel_border,
-				button_meter_border;
-
-	GdkImlibImage
-				*decal_misc_image;
-	GdkPixmap	*decal_misc_pixmap;
-	GdkBitmap	*decal_misc_mask;
-
-	/* These two vars should be handled in mail.c, but for historical reasons
-	|  gkrellmrcs define them in a way that must be handled in config.c
-	*/
-	gint		decal_mail_frames,
-				decal_mail_delay;
-
-
-	GdkGC		*draw1_GC,
-				*draw2_GC,
-				*draw3_GC,
-				*draw_stencil_GC,
-				*text_GC;
-
-	GdkGC		*bit1_GC,		/* Depth 1 GCs		*/
-				*bit0_GC;
-
-	GdkFont		*large_font,
-				*normal_font,
-				*small_font;
-
-	gchar		*large_font_string,
-				*normal_font_string,
-				*small_font_string;
-
-	GdkImlibImage *decal_alarm_image;
-	GdkImlibImage *decal_warn_image;
-	gint		decal_alarm_frames,
-				decal_warn_frames;
-
-	GdkImlibImage *bg_separator_image;
-	GdkPixmap	*bg_separator_pixmap;
-	gint		bg_separator_height;
-
-	GdkImlibImage *data_in_image,		/* Default data layers 0,2, ... */
-				*data_in_grid_image;
-	GdkPixmap	*data_in_pixmap,
-				*data_out_pixmap;
-
-	GdkImlibImage *data_out_image,		/* Default data layers 1,3, ... */
-				*data_out_grid_image;
-	GdkPixmap	*data_in_grid_pixmap,
-				*data_out_grid_pixmap;
-
-	gchar		*chart_in_color,
-				*chart_in_color_grid,
-				*chart_out_color,
-				*chart_out_color_grid;
-
-	GdkColor	in_color;	/* For cpu user, disk read, rx data	*/
-	GdkColor	out_color;	/* For cpu sys, disk writes, tx data	*/
-	GdkColor	in_color_grid;
-	GdkColor	out_color_grid;
-	GdkColor	background_color;
-	GdkColor	white_color;
-
-	gint		bg_grid_mode;
-	gint		theme_n_alternatives;
-	gint		theme_alternative;
-	gboolean	font_override,
-				use_fontset;
-	gint		m2;
-	gint		use_top_bottom_margins;
-
-	GList		*chart_name_list,		/* Move these lists to monitor_list */
-				*meter_name_list,
-				*custom_name_list,
-				*bg_chart_image_list,
-				*bg_grid_image_list,
-				*bg_panel_image_list,
-				*bg_meter_image_list,
-				*krell_panel_image_list,
-				*krell_meter_image_list,
-				*chart_style_list,
-				*panel_style_list,
-				*meter_style_list,
-				*custom_style_list;
-	};
-
-  /* Debugs for debug_level	*/
-#define DEBUG_MAIL		0x10
-#define DEBUG_NET		0x20
-#define DEBUG_TIMER		0x40
-#define	DEBUG_SENSORS	0x80
-
-
-extern struct ThemeConfig  GK;
+extern GkrellmTicks	GK;
 
 
 /* ================= User Config ==================*/
 
-  /* Values for Monitor id
+  /* Values for GkrellmMonitor id
   |  Give an id number to all builtin monitors
   |				____ cccc ___i iiii
   */
@@ -984,7 +971,8 @@ extern struct ThemeConfig  GK;
 #define	MON_MEM		5
 #define	MON_FS		6
 #define	MON_MAIL	7
-#define	MON_APM		8
+#define	MON_BATTERY	8
+#define	MON_APM		MON_BATTERY
 #define	MON_UPTIME	9
 #define	MON_CLOCK	10
 #define	MON_CAL		11
@@ -1011,17 +999,18 @@ extern struct ThemeConfig  GK;
 #define	MONITOR_ID(m)			((m)->id & MON_ID_MASK)
 #define	MONITOR_CONFIG(m,flag)	(((m)->id & MON_CONFIG_MASK) & flag)
 
-
+#define	MONITOR_ENABLED(mon)	((mon)->privat->enabled)
 
 typedef struct
 	{
 	GtkWidget		*main_vbox,
-					*spacer_top_vbox,
-					*vbox,
-					*spacer_bottom_vbox,
-					*notebook_config_widget;
-	gint			notebook_config_page;
-	GtkCTreeNode	*ctree_config_node;
+					*vbox;
+
+	gint			config_page;
+	GtkTreeRowReference	*row_reference;
+	GtkWidget		*config_vbox;
+	gboolean		config_created;
+
 	void			(*cb_disable_plugin)(void);
 	gint			insert_before_id,
 					gravity,
@@ -1030,28 +1019,25 @@ typedef struct
 	gboolean		enabled,
 					created,
 					from_command_line,
-					spacers_shown;
-	Style			*panel_style,
+					spacers_shown,
+					cap_images_off,
+					spacer_overlap_off,
+					instant_apply;
+	GkrellmStyle	*panel_style,
 					*chart_style;
 	gchar			*style_name;
 	gint			style_type,
 					style_id;	/* helper until I consolidate image lists */
-	GdkImlibImage	*bg_chart_image,
-					*bg_panel_image,
-					*bg_grid_image,
-					*krell_image;
-	GdkImlibImage	*spacer_top_image,
-					*spacer_bottom_image;
-	GtkWidget		*top_pixmapwid,
-					*bottom_pixmapwid;
-	gint			spacer_top_height,
-					spacer_bottom_height;
+	gint			top_type,
+					bottom_type;
+	GkrellmSpacer	top_spacer,
+					bottom_spacer;
 	}
-	MonPrivate;
+	GkrellmMonprivate;
 
 
 
-typedef struct
+typedef struct _GkrellmMonitor
 	{
 	gchar		*name;
 	gint		id;
@@ -1066,16 +1052,16 @@ typedef struct
 	
 	void		(*undef2)(void);
 	void		(*undef1)(void);
-	MonPrivate	*privat;
+	GkrellmMonprivate *privat;
 
 	gint		insert_before_id;		/* If plugin, insert before this mon*/
 	
 	void		*handle;				/* If monitor is a plugin.	*/
 	gchar		*path;					/* 	"						*/
 	}
-	Monitor;
+	GkrellmMonitor;
 
 extern GList	*gkrellm_monitor_list;
 
 
-#include "gkrellm_public_proto.h"
+#include "gkrellm-public-proto.h"
