@@ -292,6 +292,13 @@ decal_text_list_free(GList **text_list)
 	}
 
 
+  /* Draw to first GkrellmText in text_list and truncate text_list to make
+  |  gkrellm_draw_decal_text() compatible with plugin code written before
+  |  the text_list implementation.  For backwards compatibility, d->x_off
+  |  shadows the first tx->x_off.
+  |  Combining gkrellm_draw_decal_text() and gkrellm_decal_text_insert()
+  |  functions on the same decal may lead to unexpected results.
+  */
 static void
 gkrellm_draw_decal_internal(GkrellmPanel *p, GkrellmDecal *d, gchar *s,
 			gint value)
@@ -334,6 +341,42 @@ gkrellm_draw_decal_markup(GkrellmPanel *p, GkrellmDecal *d, gchar *s)
 	gkrellm_draw_decal_internal(p, d, s, 0);
 	}
 
+void
+gkrellm_decal_text_get_offset(GkrellmDecal *d, gint *x, gint *y)
+	{
+	if (!d)
+		return;
+	if (x)
+		*x = d->x_off;
+	if (y)
+		*y = d->y_off;
+	}
+
+  /* Setting offset of text decal means setting offset of first GkrellmText
+  |  in text_list.  For backwards compatibility with plugin code that
+  |  directly accesses d->x_off, shadow d->x_off to first tx->x_off.
+  */
+void
+gkrellm_decal_text_set_offset(GkrellmDecal *d, gint x, gint y)
+	{
+	GList		*list;
+	GkrellmText	*tx;
+
+	if (!d || (d->x_off == x && d->y_off == y))
+		return;
+
+	d->x_off = x;
+	d->y_off = y;
+	if ((list = d->text_list) != NULL)
+		{
+		tx = (GkrellmText *) list->data;
+		tx->x_off = x;
+		tx->x_off_prev = x;
+		tx->y_off = y - d->y_ink;;
+		}
+	d->modified = TRUE;
+	}
+
 void
 gkrellm_decal_text_clear(GkrellmDecal *d)
 	{
@@ -357,6 +400,10 @@ gkrellm_decal_text_clear(GkrellmDecal *d)
 	d->modified = TRUE;
 	}
 
+  /* Insert text into a decal's text_list at a given offset.  Using text
+  |  insert functions makes d->x_off and d->y_off meaningless since offsets
+  |  are in each GkrellmText's tx->x_off and tx->y_off.
+  */
 void
 gkrellm_decal_text_insert(GkrellmDecal *d, gchar *s, GkrellmTextstyle *ts,
 		gint x_off, gint y_off)
@@ -365,6 +412,7 @@ gkrellm_decal_text_insert(GkrellmDecal *d, gchar *s, GkrellmTextstyle *ts,
 
 	if (!s || !d || d->state == DS_INVISIBLE)
 		return;
+	d->flags &= ~DF_TEXT_USE_MARKUP;
 	if (!ts)
 		ts = &d->text_style;
 	if (   d->text_list
@@ -388,35 +436,71 @@ gkrellm_decal_text_insert(GkrellmDecal *d, gchar *s, GkrellmTextstyle *ts,
 	}
 
 void
-gkrellm_decal_text_get_offset(GkrellmDecal *d, gint *x, gint *y)
+gkrellm_decal_text_markup_insert(GkrellmDecal *d, gchar *s,
+		GkrellmTextstyle *ts, gint x_off, gint y_off)
 	{
-	if (!d)
+	GkrellmText	*tx = NULL;
+
+	if (!s || !d || d->state == DS_INVISIBLE)
 		return;
-	if (x)
-		*x = d->x_off;
-	if (y)
-		*y = d->y_off;
+	d->flags |= DF_TEXT_USE_MARKUP;
+	if (!ts)
+		ts = &d->text_style;
+	if (   d->text_list
+		&& !*((GkrellmText *) d->text_list->data)->text
+	   )
+		tx = (GkrellmText *) d->text_list->data;
+
+	if (!tx)
+		{
+		tx = g_new0(GkrellmText, 1);
+		d->text_list = g_list_append(d->text_list, tx);
+		}
+
+	gkrellm_dup_string(&tx->text, s);
+	tx->x_off = x_off;
+	tx->x_off_prev = x_off;
+	tx->y_off = y_off - d->y_ink;
+	tx->text_style = *ts;
+
+	d->modified = TRUE;
 	}
 
 void
-gkrellm_decal_text_set_offset(GkrellmDecal *d, gint x, gint y)
+gkrellm_decal_text_nth_inserted_set_offset(GkrellmDecal *d, gint n,
+			gint x, gint y)
 	{
 	GList		*list;
 	GkrellmText	*tx;
 
-	if (!d || (d->x_off == x && d->y_off == y))
+	if (!d || (list = g_list_nth(d->text_list, n)) == NULL)
 		return;
 
-	d->x_off = x;
-	d->y_off = y;
-	for (list = d->text_list; list; list = list->next)
+	tx = (GkrellmText *) list->data;
+	if (tx->x_off != x || tx->y_off != y)
 		{
-		tx = (GkrellmText *) list->data;
 		tx->x_off = x;
 		tx->x_off_prev = x;
 		tx->y_off = y - d->y_ink;;
+		d->modified = TRUE;
 		}
-	d->modified = TRUE;
+	}
+
+void
+gkrellm_decal_text_nth_inserted_get_offset(GkrellmDecal *d, gint n,
+			gint *x, gint *y)
+	{
+	GList		*list;
+	GkrellmText	*tx;
+
+	if (!d || (list = g_list_nth(d->text_list, n)) == NULL)
+		return;
+
+	tx = (GkrellmText *) list->data;
+	if (x)
+		*x = tx->x_off;
+	if (y)
+		*y = tx->y_off + d->y_ink;
 	}
 
 void
@@ -1157,7 +1241,7 @@ gkrellm_create_decal_pixmap(GkrellmPanel *p,
 static GkrellmDecal *
 _create_decal_text(GkrellmPanel *p, gchar *string,
 				GkrellmTextstyle *ts, GkrellmStyle *style,
-				gint x, gint y, gint w, gint h, gint y_ink)
+				gint x, gint y, gint w, gint h, gint y_ink, gboolean markup)
 	{
 	GtkWidget		*top_win;
 	GkrellmMargin	*m;
@@ -1182,8 +1266,14 @@ _create_decal_text(GkrellmPanel *p, gchar *string,
 
 	if (string && *string)
 		{
-		gkrellm_text_extents(d->text_style.font, string, strlen(string),
-				&width, &height, &baseline, &d->y_ink);
+		if (markup)
+			gkrellm_text_markup_extents(d->text_style.font,
+					string, strlen(string),
+					&width, &height, &baseline, &d->y_ink);
+		else
+			gkrellm_text_extents(d->text_style.font,
+					string, strlen(string),
+					&width, &height, &baseline, &d->y_ink);
 		d->h = height + d->text_style.effect;
 		}
 	else
@@ -1243,7 +1333,15 @@ gkrellm_create_decal_text(GkrellmPanel *p, gchar *string,
 				GkrellmTextstyle *ts, GkrellmStyle *style,
 				gint x, gint y, gint w)
 	{
-	return _create_decal_text(p, string, ts, style, x, y, w, 0, 0);
+	return _create_decal_text(p, string, ts, style, x, y, w, 0, 0, FALSE);
+	}
+
+GkrellmDecal *
+gkrellm_create_decal_text_markup(GkrellmPanel *p, gchar *string,
+				GkrellmTextstyle *ts, GkrellmStyle *style,
+				gint x, gint y, gint w)
+	{
+	return _create_decal_text(p, string, ts, style, x, y, w, 0, 0, TRUE);
 	}
 
 GkrellmDecal *
@@ -1251,7 +1349,7 @@ gkrellm_create_decal_text_with_height(GkrellmPanel *p,
 				GkrellmTextstyle *ts, GkrellmStyle *style,
 				gint x, gint y, gint w, gint h, gint y_ink)
 	{
-	return _create_decal_text(p, NULL, ts, style, x, y, w, h, y_ink);
+	return _create_decal_text(p, NULL, ts, style, x, y, w, h, y_ink, FALSE);
 	}
 
 void
